#include <WiFi.h>
#include "wifiMng.h"

#define L1 32

String szMac;
WiFiServer server(80);

void setup() {
  Serial.begin(112500);
  pinMode(L1, OUTPUT);
  delay(1000);

  xTaskCreate(
    taskWifiConnect,          /* Task function. */
    "TaskOne",        /* String with name of task. */
    10000,            /* Stack size in bytes. */
    NULL,             /* Parameter passed as input of the task */
    1,                /* Priority of the task. */
    NULL);            /* Task handle. */

  xTaskCreate(
    taskTwo,          /* Task function. */
    "TaskTwo",        /* String with name of task. */
    10000,            /* Stack size in bytes. */
    NULL,             /* Parameter passed as input of the task */
    1,                /* Priority of the task. */
    NULL);            /* Task handle. */
}

void loop() {
  WiFiClient client = server.available();   // listen for incoming clients

  if (client) {                             // if you get a client,
    Serial.println("New Client.");           // print a message out the serial port
    String currentLine = "";                // make a String to hold incoming data from the client
    while (client.connected()) {            // loop while the client's connected
      if (client.available()) {             // if there's bytes to read from the client,
        char c = client.read();             // read a byte, then
        Serial.write(c);                    // print it out the serial monitor
        if (c == '\n') {                    // if the byte is a newline character

          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request, so send a response:
          if (currentLine.length() == 0) {
            // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
            // and a content-type so the client knows what's coming, then a blank line:
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println();

            // the content of the HTTP response follows the header:
            client.print("Click <a href=\"/H\">here</a> to turn the LED 1 on<br>\n");
            client.print("Click <a href=\"/L\">here</a> to turn the LED 1 off<br><br>\n");
            client.print("Click <a href=\"/M\">here</a> Manual<br>\n");
            client.print("Click <a href=\"/m\">here</a> Power LED on<br>\n");
            client.print("Click <a href=\"/f\">here</a> Power LED off<br><br>\n");
            client.print("Click <a href=\"/A\">here</a> Light ON + Timer 2<br>\n");


            // The HTTP response ends with another blank line:
            client.println();
            // break out of the while loop:
            break;
          } else {    // if you got a newline, then clear currentLine:
            currentLine = "";
          }
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
          //Serial.println("currentLine:");
          //Serial.println(currentLine);
        }
        // Check to see if the client request was "GET /H" or "GET /L":
        if (currentLine.endsWith("GET /H")) {
          digitalWrite(L1, HIGH);               // GET /H turns the LED on
        }
        if (currentLine.endsWith("GET /L")) {
          digitalWrite(L1, LOW);                // GET /L turns the LED off
        }
        /*
          if (currentLine.endsWith("GET /M")) {
          vPeticioModBus(MB_MANUAL);
          vDelayESP(MINIMUM_DELAY_ALLOWING_MODBUS_COMMUNICATION); // Minimum delay allowing ModBus communication
          //bReadingModBus(szResposta);
          bReadMB = true;
          }
          if (currentLine.endsWith("GET /m")) {
          vPeticioModBus(MB_POWER_LED_ON);
          vDelayESP(MINIMUM_DELAY_ALLOWING_MODBUS_COMMUNICATION); // Minimum delay allowing ModBus communication
          //bReadingModBus(szResposta);
          bReadMB = true;
          }
          if (currentLine.endsWith("GET /f")) {
          vPeticioModBus(MB_POWER_LED_OFF);
          vDelayESP(MINIMUM_DELAY_ALLOWING_MODBUS_COMMUNICATION); // Minimum delay allowing ModBus communication
          //bReadingModBus(szResposta);
          bReadMB = true;
          }
          if (currentLine.endsWith("GET /A")) {
          vPeticioModBus(MB_LIGHT_ON_TIMER2);
          vDelayESP(MINIMUM_DELAY_ALLOWING_MODBUS_COMMUNICATION); // Minimum delay allowing ModBus communication
          //bReadingModBus(szResposta);
          bReadMB = true;
          }
        */
      }
    }
    // close the connection:
    client.stop();
    Serial.println("Client Disconnected.");
  }
  vTaskDelay(10);
}

void taskWifiConnect( void * parameter )
{
  szMac = "";
  for (;;) {
    IPAddress ipA = WiFi.localIP();
    if (ipA == IPAddress(0, 0, 0, 0)) // if <--> while
    {
      setup_wifi();
      server.begin();
      szMac = szGetMac();
      ipA = WiFi.localIP();
      Serial.println("Connexió establerta");
      Serial.print("MAC: ");
      Serial.println(szMac);
      Serial.print("ip: ");
      Serial.println(ipA);
    }
    vTaskDelay(1000); // To avoid: Task watchdog got triggered. The following tasks did not reset the watchdog in time
  }
  vTaskDelete( NULL );
}

void taskTwo( void * parameter)
{
  for ( ;; ) {
    Serial.print("MAC: ");
    Serial.println(szMac);
    Serial.print("IP: ");
    if (szMac != "")
      Serial.println(WiFi.localIP());
    else
      Serial.println("0.0.0.0");
    vTaskDelay(1000);
  }
  Serial.println("Ending task 2");
  vTaskDelete( NULL );

}
